@@ -2,8 +2,10 @@
 import inspect
 import enum
 
-INDEX_PRICE_EXTRA_TRADE_DAYS = 15
-FUND_NAV_EXTRA_TRADE_DAYS = 5
+INDEX_PRICE_EXTRA_TRADE_DAYS = 8
+INDEX_VAL_EXTRA_TRADE_DAYS = 8
+FUND_NAV_EXTRA_TRADE_DAYS = 8
+FUTURE_PRICE_EXTRA_TRADE_DAYS = 8
 QUARTER_UPDATE_DATE_LIST = ('0331', '0630', '0930', '1231')
 SEMI_UPDATE_DATE_LIST = ('0630', '1231')
 
@@ -403,3 +405,10 @@ class FOFRiskType(ReadableEnum):
     R3 = 3
     R4 = 4
     R5 = 5
+
+
+class DividendCalcMethod(ReadableEnum):
+    '''分红计算方式'''
+
+    BY_TOTAL_AMOUNT = 1  # 红利总额
+    BY_PER_UNIT = 2  # 每单位红利
