@@ -262,11 +262,13 @@ class DerivedDataApi(metaclass=Singleton):
                 desc_dic = index_info.set_index('index_id').to_dict()['desc_name']
                 df = df.pivot_table(columns='index_id',values=value_col,index='datetime')
                 _df = df.copy()
+                _df = _df.replace(0,np.nan)
                 df.loc['中位数',:] = _df.median()
                 df.loc['最大值',:] = _df.max()
                 df.loc['最小值',:] = _df.min()
                 df.loc['现值',:] = _df.ffill().iloc[-1]
                 df = df.rename(columns=desc_dic)
+                
                 df = df.loc[['中位数','最大值','最小值','现值']].round(2)   
                 cols = df.columns.tolist()
                 name_dic = {i:i.replace('(申万)','') for i in cols}
@@ -702,15 +704,15 @@ class DerivedDataApi(metaclass=Singleton):
             except Exception as e:
                 print(f'failed to get_fof_nav <err_msg> {e} from {FOFNav.__tablename__}')
 
-    def delete_fof_nav(self, date_to_delete: datetime.date, manager_id: str, fof_id_list: List[str]) -> bool:
+    def delete_fof_nav(self, manager_id: str, fof_id: str, date_list: List[datetime.date]) -> bool:
         with DerivedDatabaseConnector().managed_session() as session:
             try:
                 session.query(
                     FOFNav
                 ).filter(
-                    FOFNav.fof_id.in_(fof_id_list),
-                    FOFNav.datetime == date_to_delete,
                     FOFNav.manager_id == manager_id,
+                    FOFNav.fof_id == fof_id,
+                    FOFNav.datetime.in_(date_list),
                 ).delete(synchronize_session=False)
                 session.commit()
                 return True
@@ -766,6 +768,38 @@ class DerivedDataApi(metaclass=Singleton):
                 print(f'failed to delete data <err_msg> {e} from {FOFNavCalc.__tablename__}')
                 return False
 
+    def get_fof_nav_unconfirmed(self, manager_id, fof_id_list: Tuple[str] = ()):
+        with DerivedDatabaseConnector().managed_session() as session:
+            try:
+                query = session.query(
+                    FOFUnconfirmedNav,
+                ).filter(
+                    FOFUnconfirmedNav.manager_id == manager_id,
+                )
+                if fof_id_list:
+                    query = query.filter(
+                        FOFUnconfirmedNav.fof_id.in_(fof_id_list),
+                    )
+                return pd.read_sql(query.statement, query.session.bind)
+            except Exception as e:
+                print(f'failed to get_fof_nav_unconfirmed <err_msg> {e} from {FOFUnconfirmedNav.__tablename__}')
+
+    def delete_fof_nav_unconfirmed(self, manager_id: str, fof_id: str, date_list: List[datetime.date]) -> bool:
+        with DerivedDatabaseConnector().managed_session() as session:
+            try:
+                session.query(
+                    FOFUnconfirmedNav
+                ).filter(
+                    FOFUnconfirmedNav.manager_id == manager_id,
+                    FOFUnconfirmedNav.fof_id == fof_id,
+                    FOFUnconfirmedNav.datetime.in_(date_list),
+                ).delete(synchronize_session=False)
+                session.commit()
+                return True
+            except Exception as e:
+                print(f'failed to delete data <err_msg> {e} from {FOFUnconfirmedNav.__tablename__}')
+                return False
+
     def get_fof_position(self, manager_id: str, fof_id_list: Tuple[str] = ()):
         with DerivedDatabaseConnector().managed_session() as session:
             try:
