@@ -150,9 +150,9 @@ class PortfolioDataApi(metaclass=Singleton):
     def get_all_fund_nav(self, start_date=None, end_date=None, fund_list: Tuple[str] = ()):
         try:
             if start_date is None:
-                start_date = ''
+                _start_date = ''
             else:
-                start_date = start_date-datetime.timedelta(days=20)
+                _start_date = start_date-datetime.timedelta(days=20)
             end_date = end_date if end_date is not None else ''
             mutual_fund_nav = pd.DataFrame()
             hedge_fund_nav = pd.DataFrame
@@ -161,12 +161,12 @@ class PortfolioDataApi(metaclass=Singleton):
                 mutual_fund_list = [_ for _ in fund_list if '!' in _]
                 hedge_fund_list = [_ for _ in fund_list if '!' not in _]
                 if len(mutual_fund_list) > 0:
-                    mutual_fund_nav = BasicDataApi().get_fund_nav_with_date(start_date=start_date,end_date=end_date,fund_list=mutual_fund_list)
+                    mutual_fund_nav = BasicDataApi().get_fund_nav_with_date(start_date=_start_date,end_date=end_date,fund_list=mutual_fund_list)
                 if len(hedge_fund_list) > 0:
-                    hedge_fund_nav = RawDataApi().get_hf_fund_nav(fund_ids=hedge_fund_list, start_date=start_date, end_date=end_date)
+                    hedge_fund_nav = RawDataApi().get_hf_fund_nav(fund_ids=hedge_fund_list, start_date=_start_date, end_date=end_date)
             else:
-                mutual_fund_nav = BasicDataApi().get_fund_nav_with_date(start_date=start_date,end_date=end_date)
-                hedge_fund_nav = RawDataApi().get_hf_fund_nav_dt(start_date=start_date, end_date=end_date)
+                mutual_fund_nav = BasicDataApi().get_fund_nav_with_date(start_date=_start_date,end_date=end_date)
+                hedge_fund_nav = RawDataApi().get_hf_fund_nav_dt(start_date=_start_date, end_date=end_date)
             
             if not mutual_fund_nav.empty:
                 mutual_fund_nav = mutual_fund_nav.pivot_table(index='datetime',columns='fund_id',values='adjusted_net_value')
@@ -239,8 +239,8 @@ class PortfolioDataApi(metaclass=Singleton):
             for dts in date_pairs:
                 b_d = dts[0]
                 e_d = dts[1]
-                
-                fund_wgt_dt = pd.Series({k : v for k, v in order_dic[b_d].items() if v > 0})
+                existed_fund_nav_list = list(fund_nav.loc[b_d:].iloc[0].dropna().keys())
+                fund_wgt_dt = pd.Series({k : v for k, v in order_dic[b_d].items() if v > 0 and k in existed_fund_nav_list})
                 fund_wgt_dt = fund_wgt_dt / fund_wgt_dt.sum()
                 fund_amt_dt = port_amount * fund_wgt_dt
                 fund_nav_dt = fund_nav.loc[b_d:e_d,fund_wgt_dt.keys()]
@@ -323,6 +323,7 @@ class PortfolioDataApi(metaclass=Singleton):
             fund_nav = fund_nav.dropna()
             fund_nav.columns=['组合']
             basic = BasicDataApi()
+            # TODO if change asset_price
             index_price = basic.get_index_price_dt(index_list=[index_id],start_date=fund_nav.index[0] - datetime.timedelta(days=10),end_date=fund_nav.index[-1]).pivot_table(index='datetime',columns='index_id',values='close')
             index_info = basic.get_index_info([index_id])
             name_dic = index_info.set_index('index_id').to_dict()['desc_name']
@@ -433,8 +434,10 @@ class PortfolioDataApi(metaclass=Singleton):
             fre = result[0].tolist()
             _sum = np.sum(fre)
             fre = [ i / _sum for i in fre]
+            rets = result[1].round(3).tolist()
+            rets = [f'[{round(i*100,2)}%,{round(rets[idx+1]*100,2)}%)' for idx, i in enumerate(rets) if i != rets[-1]]
             data = {'分布_频度':fre,
-                '分布_收益':result[1].round(3).tolist(),
+                '分布_收益':rets,
                 '收益':ret
             }
             return data
@@ -456,6 +459,7 @@ class PortfolioDataApi(metaclass=Singleton):
             fund_nav = fund_nav.dropna()
             fund_nav.columns=['组合']
             basic = BasicDataApi()
+            # TODO if change asset_price
             index_price = basic.get_index_price_dt(index_list=[index_id],start_date=fund_nav.index[0] - datetime.timedelta(days=10),end_date=fund_nav.index[-1]).pivot_table(index='datetime',columns='index_id',values='close')
             index_info = basic.get_index_info([index_id])
             name_dic = index_info.set_index('index_id').to_dict()['desc_name']
@@ -467,7 +471,7 @@ class PortfolioDataApi(metaclass=Singleton):
                 res_status = Calculator.get_benchmark_stat_result(dates=df.index,
                                                                 values=df[col],
                                                                 benchmark_values=df[name_dic[index_id]],
-                                                                frequency = '1W',
+                                                                frequency = '1D',
                                                                 risk_free_rate=0.025, )
                 
                 indicators_tem={
@@ -611,6 +615,7 @@ class PortfolioDataApi(metaclass=Singleton):
                 }
                 return data
             fund_nav = fund_nav.ffill()
+            # TODO if change asset_price
             index_price = basic.get_index_price(index_list=[index_id])
             index_price = index_price.pivot_table(columns='index_id',index='datetime',values='close')
             nav_df = fund_nav.join(index_price).ffill().dropna()
@@ -852,6 +857,7 @@ class PortfolioDataApi(metaclass=Singleton):
             dts_list = [j for i in order_list for j in i.keys()] + [fund_nav.index[-1]]
             dts_list = [[i,dts_list[dts_list.index(i)+1]] for i in dts_list if i != dts_list[-1]]
 
+            # TODO if change asset_price
             index_price = BasicDataApi().get_index_price_dt(index_list=list(benchmark_id_info.values()),
                                                             start_date=fund_nav.index[0],
                                                             end_date=fund_nav.index[-1],
@@ -914,15 +920,19 @@ class PortfolioDataApi(metaclass=Singleton):
                 '个基近期收益':df,
             }
             # 取非0权重大类
-            fund_wgt = result['各基权重'].tail()
+            fund_wgt = result['各基权重']
             _result = []
             for type_i, fund_list in fund_type.items():
-                _df = fund_wgt[fund_list].sum(axis=1)
+                _df = fund_wgt.tail()[fund_list].sum(axis=1)
                 _df.name = type_i
                 _result.append(_df)
             fund_type_wgt = pd.concat(_result,axis=1)
             fund_type_wgt = fund_type_wgt.tail(1).reset_index(drop=True).T
-            exsited_index_list = fund_type_wgt[fund_type_wgt>0].dropna().index.tolist()
+
+            exchange_list = [j for i in order_list for j in list(i.keys()) ]
+            exchange_list = np.array(exchange_list)
+            dts_list = [begin_date] + exchange_list[exchange_list>begin_date].tolist() + [end_date]
+            date_pairs = [[dt, dts_list[idx+1]] for idx, dt in enumerate(dts_list) if dt != dts_list[-1]]
 
             port_nav = PortfolioDataApi().get_portfolio_mv(fund_nav=fund_nav, index_id=index_id,begin_date=begin_date,end_date=end_date,time_para=time_para)
             fund_info = self.get_all_fund_info(fund_list = result['各基净值'].columns.tolist())
@@ -937,35 +947,47 @@ class PortfolioDataApi(metaclass=Singleton):
 
             _index_price = result['大类净值'].loc[begin_date:]
             benchmark_id_info_r = {v: k for k, v in benchmark_id_info.items()}
+            # TODO if change asset_price
             index_price = BasicDataApi().get_index_price_dt(index_list=list(benchmark_id_info.values()),start_date=begin_date,end_date=end_date)
             index_price = index_price.pivot_table(columns='index_id',values='close',index='datetime')
             index_price = index_price.rename(columns=benchmark_id_info_r)
             index_price = index_price.reindex(_index_price.index)
             result_2 = _index_price / _index_price.iloc[0] / (index_price / index_price.iloc[0])
-            # if remove not existed asset
-            #result_2 = result_2[exsited_index_list]
+
             price_ratio = result_2.copy()
             res_2_dic = (_index_price.iloc[-1] / _index_price.iloc[0] - 1) * 10000
             result_2_str = ' '.join([f'{k} {int(v)}BP' for k, v in res_2_dic.items()])
 
-            recent_ret = result['各基净值'].loc[begin_date:].dropna(axis=1)
-            recent_ret = (recent_ret.iloc[-1] / recent_ret.iloc[0] - 1).to_frame()
-            recent_ret.columns = ['基金涨跌']
-            # TODO 考虑换仓
-            wgt_df = result['各基权重'].tail(1).T
-            wgt_df.columns = ['权重']
-            recent_ret = recent_ret.join(wgt_df)
-            recent_ret = recent_ret.dropna()
-            recent_ret.loc[:,'基金收益'] = recent_ret['基金涨跌'] * recent_ret['权重']
-            recent_ret['基金涨跌'] = recent_ret['基金涨跌'].map(lambda x : str(round(x * 100,2))+'%')
-            recent_ret['权重'] = recent_ret['权重'].map(lambda x: str(round(100*x,2))) +'%'
-            recent_ret['基金收益'] = recent_ret['基金收益'].map(lambda x: str(int(10000*x))) +'BP'
-            recent_ret.loc[:,'开始时间'] = price_ratio.index[0]
-            recent_ret.loc[:,'结束时间'] = price_ratio.index[-1]
-            recent_ret.loc[:,'基金名'] = recent_ret.index.map(fund_desc_dic)
-            recent_ret.loc[:,'基金类型'] = recent_ret.index.map(fund_type_dic)
-            recent_ret = recent_ret[['基金名','开始时间','结束时间','基金类型','基金涨跌','权重','基金收益']]
-            result_3 = recent_ret
+            fund_navs = result['各基净值']
+            _result_3 = []
+            for dts in date_pairs:
+                b_d, e_d = dts
+                if dts != date_pairs[-1]:
+                    _fund_wgt = fund_wgt.loc[b_d:e_d].iloc[:-1]
+                else:
+                    _fund_wgt = fund_wgt.loc[b_d:e_d]
+                b_d = _fund_wgt.index[0]
+                e_d = _fund_wgt.index[-1] 
+                fund_wgt_i = _fund_wgt.dropna(axis=1).mean()
+                _df_i = fund_wgt_i.to_frame()
+                _df_i.columns=['权重']
+
+                _df_i.loc[:,'基金名'] = _df_i.index.map(fund_desc_dic)
+                _df_i.loc[:,'开始时间'] = b_d
+                _df_i.loc[:,'结束时间'] = e_d
+                _df_i.loc[:,'基金类型'] = _df_i.index.map(fund_type_dic)
+                _fund_nav_i = fund_navs.loc[b_d:e_d][_df_i.index]
+                _fund_ret = _fund_nav_i.iloc[-1] / _fund_nav_i.iloc[0] - 1
+                _df_i.loc[:,'基金涨跌'] = _fund_ret
+                _df_i.loc[:,'基金收益'] = _df_i.基金涨跌 * _df_i.权重
+                _result_3.append(_df_i)
+            result_3 = pd.concat(_result_3)
+            result_3['权重'] = result_3['权重'].map(lambda x: str(round(100*x,2))) +'%'
+            dic = (result_3[['基金类型','基金收益']].groupby('基金类型').sum()*10000).astype(int).astype(str)
+            result_2_str = ' '.join([f'{k} {int(v)}BP' for k, v in dic.to_dict()['基金收益'].items()])
+            result_3['基金收益'] = result_3['基金收益'].map(lambda x: str(int(10000*x))) +'BP'
+            result_3['基金涨跌'] = result_3['基金涨跌'].map(lambda x : str(round(x * 100,2))+'%')
+            result_3 = result_3[['基金名','开始时间','结束时间','基金类型','基金涨跌','权重','基金收益']]
             result_2 = result_2.reindex(pd.date_range(start=result_2.index[0], end=result_2.index[-1]))
             result_2.index.name = 'datetime'
             data = {
@@ -987,4 +1009,102 @@ class PortfolioDataApi(metaclass=Singleton):
                 '资产价格比收益描述':[],
                 '个基近期收益':df,
             }
-            return data
\ No newline at end of file
+            return data
+
+    def get_port_info(self, trade_history, benchmark_id_info):
+        try:
+            data = PortfolioDataApi().get_portfolio_nav(trade_history=trade_history)
+            order_list, fund_type = self.trade_history_parse(trade_history)
+            fund_nav = data['净值']
+            res_status = Calculator.get_stat_result(dates=fund_nav.index, values=fund_nav['组合'])
+            dic = {
+                'annual_ret':round(res_status.annualized_ret,4),
+                'annual_vol':round(res_status.annualized_vol,4),
+                'mdd':round(res_status.mdd,4),
+            }
+
+            fund_weight = data['各基权重'].tail(1).dropna(axis=1).T
+            fund_weight.columns = ['weight']
+            result = {}
+            for types, fund_list in fund_type.items():
+                for fund_id in fund_list:
+                    result[fund_id] = types
+            fund_weight.loc[:,'fund_type'] = fund_weight.index.map(result)
+            asset_weight = fund_weight.groupby('fund_type').sum().round(2)['weight'].to_dict()
+
+            fund_info = self.get_all_fund_info(fund_weight.index.tolist())
+            name_dic = fund_info.set_index('fund_id').to_dict()['desc_name']
+
+            asset_weight_result = []
+            for k, v in asset_weight.items():
+                _dic = {
+                    'index_id':benchmark_id_info[k],
+                    'weight':v,
+                    'asset_name':k,
+                }
+                asset_weight_result.append(_dic)
+            fund_weight.loc[:,'desc_name'] = fund_weight.index.map(name_dic)
+            fund_weight = fund_weight.sort_values('weight', ascending=False)
+
+            fund_pos_result = []
+            for r in fund_weight.itertuples():
+                _dic = {
+                    'fund_id':r.Index,
+                    'weight':round(r.weight,4),
+                    'desc_name':r.desc_name,
+                }
+                fund_pos_result.append(_dic)
+            data = {
+                'port_status':dic,
+                'port_asset_weight':asset_weight_result,
+                'port_fund_weight':fund_pos_result,
+            }
+            return data
+        except Exception as e:
+            print(f'Failed to get data <err_msg> {e} from portfolio.get_port_info')
+            df = pd.DataFrame()
+            df.index.name = 'datetime'
+            data = {
+                'port_status':{},
+                'port_asset_weight':[],
+                'port_fund_weight':[],
+            }
+            return data
+
+    def calc_port_status(self, trade_history):
+        try:
+            data = self.get_portfolio_nav(trade_history=trade_history)
+            fund_nav = data['净值']
+            p_s = Calculator.get_stat_result(dates=fund_nav.index, values=fund_nav.组合).__dict__
+            dic = {
+                '最近净值':p_s['last_unit_nav'],
+                '最近涨幅':p_s['last_increase_rate'],
+                '截止日期':str(p_s['end_date']),
+                '年化收益率':p_s['annualized_ret'],
+                '年化波动率':p_s['annualized_vol'],
+                '最大回撤':p_s['mdd'],
+                '夏普率':p_s['sharpe'],
+                '卡玛比率':p_s['ret_over_mdd'],
+                '最近一年收益率':p_s['recent_y1_ret'],
+                '最近三年收益率':p_s['recent_y3_ret'],
+                '最近五年收益率':p_s['recent_y5_ret'],
+            }
+            return dic
+
+        except Exception as e:
+            print(f'Failed to get data <err_msg> {e} from portfolio.calc_port_status')
+            return {
+                    '最近净值':None,
+                    '最近涨幅':None,
+                    '截止日期':None,
+                    '年化收益率':None,
+                    '年化波动率':None,
+                    '最大回撤':None,
+                    '夏普率':None,
+                    '卡玛比率':None,
+                    '最近一年收益率':None,
+                    '最近三年收益率':None,
+                    '最近五年收益率':None,
+                }
+            
+
