@@ -21,6 +21,7 @@ from ...view.derived_models import FundManagerInfo, FundManagerIndex, FundManage
 from .derived_data_helper import DerivedDataHelper, normalize, score_rescale
 from ...manager.fund_info_filter import EHR_PROBLEM_EKYWORDS
 
+
 class ManagerProcessorDev:
 
     #INDEX_LIST = {'csi_boodfund': 'H11023.CSI', 'csi_f_hybrid': 'H11022.CSI', 'csi_f_qdii': 'H11026.CSI', 'csi_stockfund': 'H11021.CSI', 'mmf': 'H11025.CSI','hs300':'hs300'}
@@ -64,8 +65,8 @@ class ManagerProcessorDev:
         self.mng_indicator_list: Dict[str, pd.DataFrame] = {}  # 基金经理评分
 
     def init(self, start_date=str, end_date=str, print_time=False, is_mng_index=False, is_mng_indicator=False):
-        self.start_date = start_date if isinstance(start_date, datetime.datetime) else datetime.datetime.strptime(start_date,'%Y-%m-%d').date()
-        self.end_date = end_date if isinstance(end_date, datetime.datetime) else datetime.datetime.strptime(end_date,'%Y-%m-%d').date()
+        self.start_date = start_date if isinstance(start_date, datetime.date) else pd.to_datetime(start_date, infer_datetime_format=True).date()
+        self.end_date = end_date if isinstance(end_date, datetime.date) else pd.to_datetime(end_date, infer_datetime_format=True).date()
 
         with BasicDatabaseConnector().managed_session() as quant_session:
             # 交易日序列
@@ -366,13 +367,13 @@ class ManagerProcessorDev:
         last_trading_day = self.trading_days_list[self.trading_days_list < total_ret.index.values[0]].values[-1]
         total_ret.loc[last_trading_day] = 0
         total_ret = total_ret.sort_index()
-        
+
         # 做指数
         fund_manager_index = np.exp(total_ret.cumsum())
 
         # 基金经理代表作
         manager_best_fund = pd.DataFrame((fund_weight + manager_status.cumsum()/1e8).idxmax(axis=1), columns=[m_id])
-        self.mng_best_fund[fund_type].loc[:, m_id] = manager_best_fund
+        self.mng_best_fund[fund_type][m_id] = manager_best_fund
         return fund_manager_index
 
     def calcualtor(self, df, period='monthly'):
@@ -572,7 +573,7 @@ class ManagerProcessorDev:
     def process(self, end_date):
         failed_tasks = []
         try:
-            start_date = '2005-01-01'
+            start_date = '20050101'
             self.init(start_date=start_date,end_date=end_date,is_mng_index=True)
             self.calc_index()
             self.calc_mng_indicator()
@@ -584,12 +585,7 @@ class ManagerProcessorDev:
             failed_tasks.append('manager_indicator_score')
         return failed_tasks
 
+
 if __name__ == '__main__':
-    self = ManagerProcessorDev(DerivedDataApi())
-    start_date = '2005-01-01'
-    end_date = '2020-10-31'
-    self.init(start_date=start_date,end_date=end_date,is_mng_index=True)
-    self.calc_index()
-    self.calc_mng_indicator()
-    self.calc_score()
-    self.to_s3()
\ No newline at end of file
+    mpd = ManagerProcessorDev(DerivedDataHelper())
+    mpd.process('20210226')
