@@ -20,7 +20,7 @@ from ...constant import HoldingAssetType, FOFTradeStatus
 from ..api.basic import BasicDataApi
 from ..api.derived import DerivedDataApi
 from ..view.basic_models import HedgeFundInfo, FOFInfo, HedgeFundNAV, FOFInvestorPosition, FOFInvestorPositionSummary
-from ..view.derived_models import FOFNavCalc, FOFPosition, FOFInvestorData, FOFPositionDetail
+from ..view.derived_models import FOFNav, FOFNavCalc, FOFPosition, FOFInvestorData, FOFPositionDetail
 from ..wrapper.mysql import BasicDatabaseConnector, DerivedDatabaseConnector
 from ..nav_reader.hedge_fund_nav_reader import HedgeFundNAVReader
 from .manager_hedge_fund import HedgeFundDataManager
@@ -94,18 +94,19 @@ class FOFDataManagerLite():
         incentive_fee_ratio = fund_info['incentive_fee_ratio'].to_dict()
         v_nav_decimals = fund_info['v_nav_decimals'].to_dict()
 
-        df = FOFDataManagerLite.get_hedge_fund_nav(fund_list)
+        df = DerivedDataApi().get_fof_nav(manager_id, fund_list)
+        df = df.rename(columns={'fof_id': 'fund_id'})
         df = df[df.fund_id.isin(fund_info.index.array)]
 
         if asset_allocation is None:
             asset_allocation = FOFDataManagerLite.get_fof_asset_allocation(manager_id, [fof_id])
         asset_allocation = asset_allocation.loc[(asset_allocation.asset_type == HoldingAssetType.HEDGE) & (asset_allocation.fund_id.isin(fund_list)), :]
-        df = df.append(asset_allocation.loc[asset_allocation.event_type.isin([FOFTradeStatus.PURCHASE, FOFTradeStatus.SUBSCRIBE]), ['datetime', 'fund_id', 'water_line', 'nav']].rename(columns={'water_line': 'acc_unit_value', 'nav': 'net_asset_value'}))
+        df = df.append(asset_allocation.loc[asset_allocation.event_type.isin([FOFTradeStatus.PURCHASE, FOFTradeStatus.SUBSCRIBE]), ['datetime', 'fund_id', 'water_line', 'nav']].rename(columns={'water_line': 'acc_net_value'}))
         df = df.drop_duplicates(subset=['fund_id', 'datetime'])
         asset_allocation = asset_allocation.set_index('datetime')
 
         date_list: np.ndarray = pd.date_range(df.datetime.sort_values().array[0], datetime.datetime.now().date()).date
-        nav = df.pivot(index='datetime', columns='fund_id', values=['net_asset_value', 'acc_unit_value'])
+        nav = df.pivot(index='datetime', columns='fund_id', values=['nav', 'acc_net_value'])
         nav = nav.reindex(date_list).ffill().stack().reset_index(level='fund_id')
 
         v_nav_result = []
@@ -135,7 +136,7 @@ class FOFDataManagerLite():
             df = pd.DataFrame(datas_to_calc_v[row.fund_id], columns=['fund_id', 'datetime', 'unit_total', 'water_line'])
             if df.empty:
                 continue
-            v_nav = FOFDataManagerLite._do_calc_v_net_value(row.net_asset_value, df.unit_total.iloc[[-1]], row.acc_unit_value, df.water_line.iloc[[-1]], incentive_fee_ratio[row.fund_id], v_nav_decimals[row.fund_id])
+            v_nav = FOFDataManagerLite._do_calc_v_net_value(row.nav, df.unit_total.iloc[[-1]], row.acc_net_value, df.water_line.iloc[[-1]], incentive_fee_ratio[row.fund_id], v_nav_decimals[row.fund_id])
             v_nav_result.append({'datetime': row.Index, 'fund_id': row.fund_id, 'v_nav': v_nav})
         df = pd.DataFrame.from_dict(v_nav_result)
         if not df.empty:
@@ -143,14 +144,14 @@ class FOFDataManagerLite():
         return df
 
     def _calc_adj_nav_for_a_fund(df: pd.DataFrame):
-        net_asset_value = df.net_asset_value
+        net_asset_value = df.nav
         if df.shape[0] < 2:
-            return pd.Series({'datetime': df.datetime.array[-1], 'ta_factor': 1, 'adj_nav': net_asset_value.array[-1]})
+            return pd.Series({'datetime': df.datetime.array[-1], 'ta_factor': 1, 'adj_nav': net_asset_value.array[-1], 'change_rate': np.nan})
 
-        acc_unit_value = df.acc_unit_value
+        acc_unit_value = df.acc_net_value
         last_diff = round(acc_unit_value.array[-2] - net_asset_value.array[-2], 6)
         this_diff = round(acc_unit_value.array[-1] - net_asset_value.array[-1], 6)
-        assert math.isclose(this_diff - last_diff, 0) or (this_diff > last_diff), f'!!!(fund_id){df.fund_id.array[0]} (this_diff){this_diff} (last_diff){last_diff}'
+        assert math.isclose(this_diff - last_diff, 0) or (this_diff > last_diff), f'!!!(fof_id){df.fof_id.array[0]} (this_diff){this_diff} (last_diff){last_diff}'
 
         if math.isclose(last_diff, this_diff):
             ta_factor = df.ta_factor.array[-2]
@@ -160,14 +161,14 @@ class FOFDataManagerLite():
         if ta_factor is None:
             ta_factor = 1
         adj_nav = net_asset_value.array[-1] * ta_factor
-        return pd.Series({'datetime': df.datetime.array[-1], 'ta_factor': ta_factor, 'adj_nav': adj_nav})
+        return pd.Series({'datetime': df.datetime.array[-1], 'ta_factor': ta_factor, 'adj_nav': adj_nav, 'change_rate': adj_nav / df.adjusted_nav.dropna().array[-1]})
 
     @staticmethod
-    def _calc_adjusted_net_value(fund_list: List[str]):
+    def _calc_adjusted_net_value(manager_id: str, fund_list: List[str]):
         '''计算复权净值'''
 
-        df = FOFDataManagerLite.get_hedge_fund_nav(fund_list)
-        return df.groupby(by='fund_id', sort=False).apply(FOFDataManagerLite._calc_adj_nav_for_a_fund)
+        df = DerivedDataApi().get_fof_nav(manager_id, fund_list)
+        return df.groupby(by='fof_id', sort=False).apply(FOFDataManagerLite._calc_adj_nav_for_a_fund)
 
     def _init(self, manager_id: str, fof_id: str, debug_mode=False):
         def _calc_water_line_and_confirmed_nav(x):
@@ -235,12 +236,12 @@ class FOFDataManagerLite():
         # 获取私募基金净值数据
         if self._hedge_pos is not None:
             hedge_fund_list = list(self._hedge_pos.columns.unique())
-            hedge_fund_nav = FOFDataManagerLite.get_hedge_fund_nav(hedge_fund_list)
-            hedge_fund_nav = hedge_fund_nav.pivot(index='datetime', columns='fund_id').reindex(index=self._date_list)
-            self._hedge_nav = hedge_fund_nav['v_net_value']
-            self._all_latest_nav = hedge_fund_nav['net_asset_value'].ffill().iloc[-1, :]
-            self._all_latest_acc_nav = hedge_fund_nav['acc_unit_value'].ffill().iloc[-1, :]
-            self._latest_nav_date = hedge_fund_nav['net_asset_value'].apply(lambda x: x[x.notna()].index.array[-1])
+            hedge_fund_nav = DerivedDataApi().get_fof_nav(manager_id, hedge_fund_list)
+            hedge_fund_nav = hedge_fund_nav.pivot(index='datetime', columns='fof_id').reindex(index=self._date_list)
+            # self._hedge_nav = hedge_fund_nav['v_net_value']
+            self._all_latest_nav = hedge_fund_nav['nav'].ffill().iloc[-1, :]
+            self._all_latest_acc_nav = hedge_fund_nav['acc_net_value'].ffill().iloc[-1, :]
+            self._latest_nav_date = hedge_fund_nav['nav'].apply(lambda x: x[x.notna()].index.array[-1])
 
             # 我们自己算一下虚拟净值 然后拿它对_hedge_nav查缺补漏
             v_nav_calcd = FOFDataManagerLite._calc_virtual_net_value(manager_id, fof_id, hedge_fund_list, self._asset_allocation)
@@ -693,6 +694,9 @@ class FOFDataManagerLite():
             if not df.empty:
                 DerivedDataApi().delete_fof_position_detail(manager_id=manager_id, fof_id_to_delete=fof_id, fund_id_list=df[(df.manager_id == manager_id) & (df.fof_id == fof_id)].fund_id.to_list())
                 df.to_sql(FOFPositionDetail.__table__.name, DerivedDatabaseConnector().get_engine(), index=False, if_exists='append')
+            stale_fund_list = set(now_df.fund_id.array).difference(set(self._fof_position_details.fund_id.array))
+            print(f'[dump_fof_nav_and_pos_to_db] (manager_id){manager_id} (fof_id){fof_id} (stale_fund_list){stale_fund_list}')
+            DerivedDataApi().delete_fof_position_detail(manager_id=manager_id, fof_id_to_delete=fof_id, fund_id_list=stale_fund_list)
             print('[dump_fof_nav_and_pos_to_db] dump position detail done')
         else:
             print('[dump_fof_nav_and_pos_to_db] no position detail, should calc it first')
@@ -810,7 +814,7 @@ class FOFDataManagerLite():
 
     def _gather_asset_info_of_fof(self, manager_id, fof_id, fund_list: List[str], v_nav: pd.DataFrame):
         fof_aa = FOFDataManagerLite.get_fof_asset_allocation(manager_id, [fof_id])
-        hedge_nav = FOFDataManagerLite.get_hedge_fund_nav(fund_list)
+        hedge_nav = DerivedDataApi().get_fof_nav(manager_id, fund_list)
         trading_day = BasicDataApi().get_trading_day_list(start_date=hedge_nav.datetime.min(), end_date=hedge_nav.datetime.max())
         hedge_info = FOFDataManagerLite.get_hedge_fund_info(fund_list).set_index('fund_id')
         for fund_id in fund_list:
@@ -822,9 +826,9 @@ class FOFDataManagerLite():
                 redempt_aa = fund_fof_aa[fund_fof_aa.event_type.isin([FOFTradeStatus.REDEEM, ])]
                 aa_insentive_info = fund_fof_aa[fund_fof_aa.event_type.isin([FOFTradeStatus.DEDUCT_REWARD, FOFTradeStatus.DEDUCT_REWARD_AND_DIVIDEND_CASH])]
                 aa_dividend_reinvestment = fund_fof_aa[fund_fof_aa.event_type.isin([FOFTradeStatus.DIVIDEND_VOLUME, FOFTradeStatus.DEDUCT_REWARD_AND_DIVIDEND_VOLUME])]
-                fund_nav = hedge_nav[hedge_nav.fund_id == fund_id]
+                fund_nav = hedge_nav[hedge_nav.fof_id == fund_id]
                 # TODO 这里最好应该用复权净值
-                fund_auv = fund_nav.set_index('datetime')['acc_unit_value']
+                fund_auv = fund_nav.set_index('datetime')['acc_net_value']
                 ret_date_with_last_pub = {}
                 if fund_auv.size > 2:
                     ret_date_with_last_pub['date'] = fund_auv.index.array[-2]
@@ -836,7 +840,7 @@ class FOFDataManagerLite():
                 fund_data = {
                     'start_date': res_status.start_date.isoformat(),
                     'days': (fund_auv.index.array[-1] - res_status.start_date).days,
-                    'nav': round(fund_nav.net_asset_value.array[-1], 4),
+                    'nav': round(fund_nav.nav.array[-1], 4),
                     'total_ret': f'{round(total_ret * 100, 2)}%',
                     'sharpe': round(res_status.sharpe, 6),
                     'annualized_ret': f'{round(res_status.annualized_ret * 100, 2)}%',
@@ -849,9 +853,9 @@ class FOFDataManagerLite():
                 db_session = Session()
                 fof_info_to_set = db_session.query(HedgeFundInfo).filter(HedgeFundInfo.fund_id == fund_id).one_or_none()
                 fof_info_to_set.latest_cal_date = latest_nav_date
-                fof_info_to_set.net_asset_value = float(fund_nav.net_asset_value.array[-1]) if not pd.isnull(fund_nav.net_asset_value.array[-1]) else None
-                fof_info_to_set.acc_unit_value = float(fund_nav.acc_unit_value.array[-1]) if not pd.isnull(fund_nav.acc_unit_value.array[-1]) else None
-                fof_info_to_set.adjusted_net_value = float(fund_nav.adjusted_net_value.array[-1]) if not pd.isnull(fund_nav.adjusted_net_value.array[-1]) else None
+                fof_info_to_set.net_asset_value = float(fund_nav.nav.array[-1]) if not pd.isnull(fund_nav.nav.array[-1]) else None
+                fof_info_to_set.acc_unit_value = float(fund_nav.acc_net_value.array[-1]) if not pd.isnull(fund_nav.acc_net_value.array[-1]) else None
+                fof_info_to_set.adjusted_net_value = float(fund_nav.adjusted_nav.array[-1]) if not pd.isnull(fund_nav.adjusted_nav.array[-1]) else None
                 fof_info_to_set.mdd = float(res_status.mdd) if not pd.isnull(res_status.mdd) else None
                 fof_info_to_set.ret = float(total_ret) if not pd.isnull(total_ret) else None
                 fof_info_to_set.ann_ret = float(res_status.annualized_ret) if not pd.isnull(res_status.annualized_ret) else None
@@ -871,17 +875,20 @@ class FOFDataManagerLite():
                 fund_auv = fund_auv[fund_auv.index >= pur_sub_aa.sort_values(by='confirmed_date').confirmed_date.array[0]]
                 if not fund_auv.empty:
                     res_status = Calculator.get_stat_result(fund_auv.index, fund_auv.array)
-                    mv = (pur_sub_aa.share.sum() - aa_insentive_info.share.sum() + aa_dividend_reinvestment.share.sum()) * fund_nav.net_asset_value.array[-1]
-                    real_mv = mv - redempt_aa.share.sum() * fund_nav.net_asset_value.array[-1]
+                    mv = (pur_sub_aa.share.sum() - aa_insentive_info.share.sum() + aa_dividend_reinvestment.share.sum()) * fund_nav.nav.array[-1]
+                    real_mv = mv - redempt_aa.share.sum() * fund_nav.nav.array[-1]
                     total_cost = pur_sub_aa.amount.sum()
-                    v_net_value = fund_nav.v_net_value.array[-1]
-                    if not pd.isnull(v_net_value):
-                        v_net_value = round(float(v_net_value), 4)
-                    else:
+                    # v_net_value = fund_nav.v_net_value.array[-1]
+                    # if not pd.isnull(v_net_value):
+                    #     v_net_value = round(float(v_net_value), 4)
+                    # else:
+                    if v_nav is not None:
                         try:
                             v_net_value = f'{round(v_nav[fund_id].iat[-1], 4)}(计算值)'
                         except KeyError:
                             v_net_value = '/'
+                    else:
+                        v_net_value = '/'
                     if fund_nav.datetime.array[-1] > pur_sub_aa.confirmed_date.array[-1]:
                         the_xirr = f'{round(xirr(pur_sub_aa.amount.to_list() + [-mv], pur_sub_aa.confirmed_date.to_list() + [fund_nav.datetime.array[-1]]) * 100, 2)}%'
                     else:
@@ -914,23 +921,28 @@ class FOFDataManagerLite():
             print(f'[pull_hedge_fund_nav] can not found enough params in env (e){e}')
             return False
 
-        hf_nav_r = HedgeFundNAVReader(email_data_dir, user_name, password)
+        hf_nav_r = HedgeFundNAVReader(manager_id, email_data_dir, user_name, password)
         nav_df: Optional[pd.DataFrame] = hf_nav_r.read_navs_and_dump_to_db()
         if nav_df is not None:
             fund_list = list(nav_df.index.unique())
-            adj_nav = FOFDataManagerLite._calc_adjusted_net_value(fund_list)
+            adj_nav = FOFDataManagerLite._calc_adjusted_net_value(manager_id, fund_list)
             adj_nav = adj_nav[adj_nav.ta_factor.notna()]
             if not adj_nav.empty:
                 print(adj_nav)
-                Session = sessionmaker(BasicDatabaseConnector().get_engine())
+                Session = sessionmaker(DerivedDatabaseConnector().get_engine())
                 db_session = Session()
                 for row in adj_nav.itertuples():
-                    hedge_fund_nav_to_set = db_session.query(HedgeFundNAV).filter(HedgeFundNAV.fund_id == row.Index, HedgeFundNAV.datetime == row.datetime).one_or_none()
-                    hedge_fund_nav_to_set.ta_factor = row.ta_factor
-                    hedge_fund_nav_to_set.adjusted_net_value = row.adj_nav
+                    fof_nav_to_set = db_session.query(FOFNav).filter(FOFNav.fof_id == row.Index, FOFNav.datetime == row.datetime).one_or_none()
+                    fof_nav_to_set.ta_factor = row.ta_factor
+                    fof_nav_to_set.adjusted_nav = row.adj_nav
+                    # fof_nav_to_set.change_rate = row.change_rate
                     db_session.commit()
                 db_session.close()
-            v_nav = FOFDataManagerLite._calc_virtual_net_value(manager_id, fof_id, fund_list)
+            # TODO: 这里先 hard code 一下
+            if fof_id == 'SLW695':
+                v_nav = FOFDataManagerLite._calc_virtual_net_value(manager_id, fof_id, fund_list)
+            else:
+                v_nav = None
             self._gather_asset_info_of_fof(manager_id, fof_id, fund_list, v_nav)
         return True
 
