@@ -171,5 +171,5 @@ class StockDataManager(DataManager):
         pass
 
     @staticmethod
-    def get_stock_factor_ret(factor_names: Tuple[str]) -> Optional[pd.DataFrame]:
-        return StockFactorApi.get_stock_factor_ret(factor_names)
+    def get_stock_factor_ret(factor_names: Tuple[str], universe: str = 'default') -> Optional[pd.DataFrame]:
+        return StockFactorApi.get_stock_factor_ret(factor_names, universe)
