@@ -50,6 +50,16 @@ class Calculator:
         df_i = df.loc[x[0]:x[-1],][['fund','benchmark']].to_numpy()
         return Calculator._rolling_cl_alpha_beta(res, df_i)
 
+    @staticmethod
+    def rolling_tm_alpha_beta(x, res, df):
+        df_i = df.loc[x[0]:x[-1],][['fund','benchmark']].to_numpy()
+        return Calculator._rolling_tm_alpha_beta(res, df_i)
+
+    @staticmethod
+    def rolling_hm_alpha_beta(x, res, df):
+        df_i = df.loc[x[0]:x[-1],][['fund','benchmark']].to_numpy()
+        return Calculator._rolling_hm_alpha_beta(res, df_i)
+
     @staticmethod
     def rolling_mdd(x):
         # fmax nanmin ignore nan
@@ -129,6 +139,46 @@ class Calculator:
                 })
         return 1
     
+    @staticmethod
+    def _rolling_tm_alpha_beta(res, total: np.ndarray):
+        if total.shape[0] <= 1:
+            res.append({'beta':np.nan,
+                    'alpha':np.nan,
+                    })
+            return 1
+        X = np.array([total[:, 1], total[:, 1]]).T
+        X = np.array([total[:, 1], total[:, 1] * total[:, 1]]).T
+        if np.count_nonzero(X[:, 1]) == 0:
+            res.append({'beta':np.nan,
+                    'alpha':np.nan,
+                    })
+            return 1
+        est2 = sm.OLS(total[:, 0], sm.add_constant(X, prepend=False)).fit()
+        res.append({'beta':est2.params[1],
+                'alpha':est2.params[-1],
+                })
+        return 1
+
+    @staticmethod
+    def _rolling_hm_alpha_beta(res, total: np.ndarray):
+        if total.shape[0] <= 1:
+            res.append({'beta':np.nan,
+                        'alpha':np.nan,
+                        })
+            return 1
+        X = np.array([total[:, 1], total[:, 1]]).T
+        X[:, 1][X[:, 1] < 0] = 0
+        if np.count_nonzero(X[:, 1]) == 0:
+            res.append({'beta':np.nan,
+                        'alpha':np.nan,
+                        })
+            return 1
+        est2 = sm.OLS(total[:, 0], sm.add_constant(X, prepend=False)).fit()
+        res.append({'beta':est2.params[1],
+                    'alpha':est2.params[-1],
+                    })
+        return 1
+
     @staticmethod
     def data_reindex_daily_trading_day(data, tradingday):
         # 日线因子交易日对齐
