@@ -1,7 +1,9 @@
 
+from typing import Dict
+
 import pandas as pd
 
-from .factor_types import GrowthFactor
+from .factor_types import GrowthFactor, MixFactorBase
 from .basic_factors import IncomeFactor, AdjNetProfitFactor, CashFlowFactor
 from .utils import calc_growth_rate, normalize
 
@@ -10,6 +12,7 @@ class IncomeGrowthFactor(GrowthFactor):
     # 收入增长
     def __init__(self):
         super().__init__('inc_gr')
+        self._deps.add(IncomeFactor())
 
     def calc(self):
         # 过去四个季度的营业收入增长率
@@ -21,6 +24,7 @@ class NetProfitOfNonRecurringGoLFactor(GrowthFactor):
     # 扣非后盈利增长
     def __init__(self):
         super().__init__('np_nrgol')
+        self._deps.add(AdjNetProfitFactor())
 
     def calc(self):
         # 过去四个季度扣除非经常性损益后的净利润的增长率
@@ -32,6 +36,7 @@ class OperationalCashFlowFactor(GrowthFactor):
     # 经营性现金流增长
     def __init__(self):
         super().__init__('op_cf')
+        self._deps.add(CashFlowFactor())
 
     def calc(self):
         # 企业过去四个季度经营性现金流的增长率
@@ -39,18 +44,17 @@ class OperationalCashFlowFactor(GrowthFactor):
         self._factor = cash_flow.apply(calc_growth_rate, axis=1, result_type='broadcast', whole_df=cash_flow)
 
 
-class MixGrowthFactor(GrowthFactor):
+class MixGrowthFactor(MixFactorBase, GrowthFactor):
     # 合成因子
     def __init__(self):
-        super().__init__('mixg')
-        self._universe: str = 'default'
+        super(MixFactorBase, self).__init__('mixg')
+        self._factor: Dict[pd.DataFrame] = {}
+        self._deps.add(IncomeGrowthFactor())
+        self._deps.add(NetProfitOfNonRecurringGoLFactor())
+        self._deps.add(OperationalCashFlowFactor())
 
-    # TODO: 这个接口写的不好，暂时先这样
-    def set_universe(self, universe: str):
-        self._universe = universe
-
-    def calc(self):
-        index = IncomeGrowthFactor().get(self._universe).index
-        columns = IncomeGrowthFactor().get(self._universe).columns
-        self._factor = pd.DataFrame((normalize(IncomeGrowthFactor().get(self._universe).T) + normalize(NetProfitOfNonRecurringGoLFactor().get(self._universe).T) +
-                                    normalize(OperationalCashFlowFactor().get(self._universe).T)).T, index=index, columns=columns)
+    def calc(self, universe: str):
+        index = IncomeGrowthFactor().get(universe).index
+        columns = IncomeGrowthFactor().get(universe).columns
+        self._factor[universe] = pd.DataFrame((normalize(IncomeGrowthFactor().get(universe).T) + normalize(NetProfitOfNonRecurringGoLFactor().get(universe).T) +
+                                               normalize(OperationalCashFlowFactor().get(universe).T)).T, index=index, columns=columns)
