@@ -145,7 +145,7 @@ class CalculatorBase:
         recent_5y_ret = _res['y5']
         history_ret = sr.loc[:last_date][-1] / sr[0] - 1
         days = (dates[-1] - dates[0]).days
-        history_annual_ret = math.pow(history_ret + 1, CalculatorBase.TOTAL_DAYS_PER_YEAR/days) - 1
+        history_annual_ret = math.pow(history_ret + 1, CalculatorBase.TOTAL_DAYS_PER_YEAR/days) - 1 if days != 0 else 0
         sr_week = CalculatorBase.data_resample_weekly_nav(sr,rule='W-FRI')
         if sr_week.shape[0] < 5:
             recent_4w_ret = sr_week[-1] / sr_week[0] - 1
@@ -674,7 +674,7 @@ class CalculatorBase:
                     'start_date':dates[0] if len(dates) > 0 else None,
                     'end_date':dates[-1] if len(dates) > 0 else None,
                     'trade_year':0,
-                    'last_unit_nav':0,
+                    'last_unit_nav':None,
                     'cumu_ret':0,
                     'annual_ret':0,
                     'annual_vol':0,
